Package: gsignal
Type: Package
Title: Signal Processing
Version: 0.3-7
Date: 2024-09-11
Authors@R: 
    c(person("Geert", "van Boxtel", role = c("aut", "cre"),
             email = "G.J.M.vanBoxtel@gmail.com",
             comment = "Maintainer"),
      person("Tom", "Short", role = "aut",
             email = "tshort@eprisolutions.com",
             comment="Author of 'signal' package"),
      person("Paul", "Kienzle", role = "aut",
             comment = "Majority of the original sources"),
      person("Ben", "Abbott", role = "ctb"),
      person("Juan", "Aguado", role = "ctb"),
      person("Muthiah", "Annamalai", role = "ctb"),
      person("Leonardo", "Araujo", role = "ctb"),
      person("William", "Asquith", role = "ctb"),
      person("David", "Bateman", role = "ctb"),
      person("David", "Billinghurst", role = "ctb"),
      person("Juan Pablo", "Carbajal", role = "ctb"),
      person("André", "Carezia", role = "ctb"),
      person("Vincent", "Cautaerts", role = "ctb"),
      person("Eric", "Chassande-Mottin", role = "ctb"),
      person("Luca", "Citi", role = "ctb"),
      person("Dave", "Cogdell", role = "ctb"),
      person("Carlo", "de Falco", role = "ctb"),
      person("Carne", "Draug", role = "ctb"),
      person("Pascal", "Dupuis", role = "ctb"),
      person("John W.", "Eaton", role = "ctb"),
      person("R.G.H", "Eschauzier", role = "ctb"),
      person("Andrew", "Fitting", role = "ctb"),
      person("Alan J.", "Greenberger", role = "ctb"),
      person("Mike", "Gross", role = "ctb"),
      person("Daniel", "Gunyan", role = "ctb"),
      person("Kai", "Habel", role = "ctb"),
      person("Kurt", "Hornik", role = "ctb"),
      person("Jake", "Janovetz", role = "ctb"),
      person("Alexander", "Klein", role = "ctb"),
      person("Peter V.", "Lanspeary", role = "ctb"),
      person("Bill", "Lash", role = "ctb"),
      person("Friedrich", "Leissh", role = "ctb"),
      person("Laurent S.", "Mazet", role = "ctb"),
      person("Mike", "Miller", role = "ctb"),
      person("Petr", "Mikulik", role = "ctb"),
      person("Paolo", "Neis", role = "ctb"),
      person("Georgios", "Ouzounis", role = "ctb"),
      person("Sylvain", "Pelissier", role = "ctb"),
      person("Francesco", "Potortì", role = "ctb"),
      person("Charles", "Praplan", role = "ctb"),
      person("Lukas F.", "Reichlin", role = "ctb"),
      person("Tony", "Richardson", role = "ctb"),
      person("Asbjorn", "Sabo", role = "ctb"),
      person("Thomas", "Sailer", role = "ctb"),
      person("Rolf", "Schirmacher", role = "ctb"),
      person("Rolf", "Schirmacher", role = "ctb"),
      person("Ivan", "Selesnick", role = "ctb"),
      person("Julius O.", "Smith III", role = "ctb"),
      person("Peter L.", "Soendergaard", role = "ctb"),
      person("Quentin", "Spencer", role = "ctb"),
      person("Doug", "Stewart", role = "ctb"),
      person("P.", "Sudeepam", role = "ctb"),
      person("Stefan", "van der Walt", role = "ctb"),
      person("Andreas", "Weber", role = "ctb"),
      person("P.", "Sudeepam", role = "ctb"),
      person("Andreas", "Weingessel", role = "ctb"))
Description: R implementation of the 'Octave' package 'signal', containing
    a variety of signal processing tools, such as signal generation and
    measurement, correlation and convolution, filtering, filter design,
    filter analysis and conversion, power spectrum analysis, system
    identification, decimation and sample rate change, and windowing.
Depends: R (>= 3.5.0)
LinkingTo: Rcpp
Imports: pracma, Rcpp, grDevices
License: GPL-3
Encoding: UTF-8
Language: en-US
URL: https://github.com/gjmvanboxtel/gsignal
BugReports: https://github.com/gjmvanboxtel/gsignal/issues
LazyData: true
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, testthat, ggplot2, gridExtra,
        microbenchmark, covr
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2024-09-11 12:27:07 UTC; geert
Author: Geert van Boxtel [aut, cre] (Maintainer),
  Tom Short [aut] (Author of 'signal' package),
  Paul Kienzle [aut] (Majority of the original sources),
  Ben Abbott [ctb],
  Juan Aguado [ctb],
  Muthiah Annamalai [ctb],
  Leonardo Araujo [ctb],
  William Asquith [ctb],
  David Bateman [ctb],
  David Billinghurst [ctb],
  Juan Pablo Carbajal [ctb],
  André Carezia [ctb],
  Vincent Cautaerts [ctb],
  Eric Chassande-Mottin [ctb],
  Luca Citi [ctb],
  Dave Cogdell [ctb],
  Carlo de Falco [ctb],
  Carne Draug [ctb],
  Pascal Dupuis [ctb],
  John W. Eaton [ctb],
  R.G.H Eschauzier [ctb],
  Andrew Fitting [ctb],
  Alan J. Greenberger [ctb],
  Mike Gross [ctb],
  Daniel Gunyan [ctb],
  Kai Habel [ctb],
  Kurt Hornik [ctb],
  Jake Janovetz [ctb],
  Alexander Klein [ctb],
  Peter V. Lanspeary [ctb],
  Bill Lash [ctb],
  Friedrich Leissh [ctb],
  Laurent S. Mazet [ctb],
  Mike Miller [ctb],
  Petr Mikulik [ctb],
  Paolo Neis [ctb],
  Georgios Ouzounis [ctb],
  Sylvain Pelissier [ctb],
  Francesco Potortì [ctb],
  Charles Praplan [ctb],
  Lukas F. Reichlin [ctb],
  Tony Richardson [ctb],
  Asbjorn Sabo [ctb],
  Thomas Sailer [ctb],
  Rolf Schirmacher [ctb],
  Rolf Schirmacher [ctb],
  Ivan Selesnick [ctb],
  Julius O. Smith III [ctb],
  Peter L. Soendergaard [ctb],
  Quentin Spencer [ctb],
  Doug Stewart [ctb],
  P. Sudeepam [ctb],
  Stefan van der Walt [ctb],
  Andreas Weber [ctb],
  P. Sudeepam [ctb],
  Andreas Weingessel [ctb]
Maintainer: Geert van Boxtel <G.J.M.vanBoxtel@gmail.com>
Repository: CRAN
Date/Publication: 2024-09-11 22:52:47 UTC
