# Generated by roxygen2: do not edit by hand

S3method(as.Arma,Arma)
S3method(as.Arma,Ma)
S3method(as.Arma,Sos)
S3method(as.Arma,Zpg)
S3method(as.Sos,Arma)
S3method(as.Sos,Ma)
S3method(as.Sos,Sos)
S3method(as.Sos,Zpg)
S3method(as.Zpg,Arma)
S3method(as.Zpg,Ma)
S3method(as.Zpg,Sos)
S3method(as.Zpg,Zpg)
S3method(bilinear,Arma)
S3method(bilinear,Zpg)
S3method(bilinear,default)
S3method(butter,FilterSpecs)
S3method(butter,default)
S3method(cheby1,FilterSpecs)
S3method(cheby1,default)
S3method(cheby2,FilterSpecs)
S3method(cheby2,default)
S3method(ellip,FilterSpecs)
S3method(ellip,default)
S3method(fftfilt,Ma)
S3method(fftfilt,default)
S3method(filter,Arma)
S3method(filter,Ma)
S3method(filter,Sos)
S3method(filter,Zpg)
S3method(filter,default)
S3method(filter,sgolayFilter)
S3method(filter_zi,Arma)
S3method(filter_zi,Ma)
S3method(filter_zi,Sos)
S3method(filter_zi,Zpg)
S3method(filter_zi,default)
S3method(filtfilt,Arma)
S3method(filtfilt,Ma)
S3method(filtfilt,Sos)
S3method(filtfilt,Zpg)
S3method(filtfilt,default)
S3method(filtic,Arma)
S3method(filtic,Ma)
S3method(filtic,Sos)
S3method(filtic,Zpg)
S3method(filtic,default)
S3method(freqs,Arma)
S3method(freqs,Ma)
S3method(freqs,Sos)
S3method(freqs,Zpg)
S3method(freqs,default)
S3method(freqz,Arma)
S3method(freqz,Ma)
S3method(freqz,Sos)
S3method(freqz,Zpg)
S3method(freqz,default)
S3method(grpdelay,Arma)
S3method(grpdelay,Ma)
S3method(grpdelay,Sos)
S3method(grpdelay,Zpg)
S3method(grpdelay,default)
S3method(iirlp2mb,Arma)
S3method(iirlp2mb,Sos)
S3method(iirlp2mb,Zpg)
S3method(iirlp2mb,default)
S3method(impinvar,Arma)
S3method(impinvar,default)
S3method(impz,Arma)
S3method(impz,Ma)
S3method(impz,Sos)
S3method(impz,Zpg)
S3method(impz,default)
S3method(invimpinvar,Arma)
S3method(invimpinvar,default)
S3method(plot,ar_psd)
S3method(plot,grpdelay)
S3method(plot,pwelch)
S3method(plot,specgram)
S3method(print,ar_psd)
S3method(print,freqs)
S3method(print,freqz)
S3method(print,grpdelay)
S3method(print,impz)
S3method(print,pwelch)
S3method(print,specgram)
S3method(print,summary.freqs)
S3method(print,summary.freqz)
S3method(sftrans,Arma)
S3method(sftrans,Zpg)
S3method(sftrans,default)
S3method(summary,freqs)
S3method(summary,freqz)
S3method(zplane,Arma)
S3method(zplane,Ma)
S3method(zplane,Sos)
S3method(zplane,Zpg)
S3method(zplane,default)
export(Arma)
export(FilterSpecs)
export(Ma)
export(Sos)
export(Zpg)
export(ar_psd)
export(arburg)
export(aryule)
export(as.Arma)
export(as.Sos)
export(as.Zpg)
export(barthannwin)
export(bartlett)
export(besselap)
export(besself)
export(bilinear)
export(bitrevorder)
export(blackman)
export(blackmanharris)
export(blackmannuttall)
export(bohmanwin)
export(boxcar)
export(buffer)
export(buttap)
export(butter)
export(buttord)
export(cceps)
export(cconv)
export(cheb)
export(cheb1ap)
export(cheb1ord)
export(cheb2ap)
export(cheb2ord)
export(chebwin)
export(cheby1)
export(cheby2)
export(chirp)
export(cl2bp)
export(clustersegment)
export(cmorwavf)
export(cohere)
export(conv)
export(conv2)
export(convmtx)
export(cplxpair)
export(cplxreal)
export(cpsd)
export(csd)
export(czt)
export(db2pow)
export(dct)
export(dct2)
export(dctmtx)
export(decimate)
export(detrend)
export(dftmtx)
export(digitrevorder)
export(diric)
export(downsample)
export(dst)
export(dwt)
export(ellip)
export(ellipap)
export(ellipord)
export(fftconv)
export(fftfilt)
export(fftshift)
export(fht)
export(filter)
export(filter2)
export(filter_zi)
export(filtfilt)
export(filtic)
export(findpeaks)
export(fir1)
export(fir2)
export(firls)
export(flattopwin)
export(fracshift)
export(freqs)
export(freqs_plot)
export(freqz)
export(freqz_plot)
export(fwhm)
export(fwht)
export(gauspuls)
export(gaussian)
export(gausswin)
export(gmonopuls)
export(grpdelay)
export(hamming)
export(hann)
export(hanning)
export(hilbert)
export(idct)
export(idct2)
export(idst)
export(ifft)
export(ifftshift)
export(ifht)
export(ifwht)
export(iirlp2mb)
export(impinvar)
export(impz)
export(imvfft)
export(interp)
export(invfreq)
export(invfreqs)
export(invfreqz)
export(invimpinvar)
export(kaiser)
export(kaiserord)
export(levinson)
export(marcumq)
export(medfilt1)
export(mexihat)
export(meyeraux)
export(morlet)
export(movingrms)
export(mpoles)
export(mscohere)
export(ncauer)
export(nuttallwin)
export(pad)
export(parzenwin)
export(pburg)
export(peak2peak)
export(peak2rms)
export(pei_tseng_notch)
export(poly)
export(polyreduce)
export(polystab)
export(postpad)
export(pow2db)
export(prepad)
export(primitive)
export(pulstran)
export(pwelch)
export(pyulear)
export(qp_kaiser)
export(rceps)
export(rectpuls)
export(rectwin)
export(remez)
export(resample)
export(residue)
export(residued)
export(residuez)
export(rms)
export(rresidue)
export(rssq)
export(sampled2continuous)
export(sawtooth)
export(schtrig)
export(sftrans)
export(sgolay)
export(sgolayfilt)
export(shanwavf)
export(shiftdata)
export(sigmoid_train)
export(sinetone)
export(sinewave)
export(sos2tf)
export(sos2zp)
export(sosfilt)
export(specgram)
export(square)
export(stft)
export(tf2sos)
export(tf2zp)
export(tfe)
export(tfestimate)
export(triang)
export(tripuls)
export(tukeywin)
export(udecode)
export(uencode)
export(ultrwin)
export(unshiftdata)
export(unwrap)
export(upfirdn)
export(upsample)
export(upsamplefill)
export(wconv)
export(welchwin)
export(wfilters)
export(wkeep)
export(xcorr)
export(xcorr2)
export(xcov)
export(zerocrossing)
export(zp2sos)
export(zp2tf)
export(zplane)
import(grDevices)
importFrom(Rcpp,sourceCpp)
useDynLib(gsignal, .registration = TRUE)
