#' @details
#'
#' The package 'gsignal' is an implementation in R of the 'Octave' package
#' 'signal'. It provides signal processing algorithms for use with R, include
#' the creation of waveforms, FIR and IIR filter design, spectral analysis,
#' Fourier and other transforms, window functions, and resampling and rate
#' changing.
#'
#' The 'Octave' package 'signal' release 1.4.1 (2019-02-08) was used to port the
#' functions to R. Note that compatibility of function parameters and return
#' values was not explicitly pursued.
#'
#' @references \url{https://wiki.octave.org/Signal_package},\cr
#'   \url{https://octave.sourceforge.io/signal/}
#'
#' @keywords internal
"_PACKAGE"
#> [1] "_PACKAGE"
