# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

conv2df <- function(a, b) {
    .Call(`_gsignal_conv2df`, a, b)
}

conv2ds <- function(a, b) {
    .Call(`_gsignal_conv2ds`, a, b)
}

conv2dv <- function(a, b) {
    .Call(`_gsignal_conv2dv`, a, b)
}

rfilter <- function(b, a, x, zi) {
    .Call(`_gsignal_rfilter`, b, a, x, zi)
}

fwht <- function(x) {
    .Call(`_gsignal_fwht`, x)
}

remez <- function(h, numtaps, numband, bands, des, weight, type, griddensity) {
    .Call(`_gsignal_remez`, h, numtaps, numband, bands, des, weight, type, griddensity)
}

rsosfilt <- function(sos, x, zi) {
    .Call(`_gsignal_rsosfilt`, sos, x, zi)
}

upfirdn <- function(x, h, p, q) {
    .Call(`_gsignal_upfirdn`, x, h, p, q)
}

